@@ -14,7 +14,7 @@ use warnings;
 our ($_SIGCHLD, $_SIGCHLD_CNT, $REAP) = (0,0,0);
 our (@CHLD_HANDLE_HISTORY, @SIGCHLD_CAUGHT) = (0);
 our $SIG_DEBUG = $ENV{SIG_DEBUG};
-our $VERSION = '0.95';
+our $VERSION = '0.96';
 my %bastards;
 
 #
@@ -58,7 +58,7 @@ sub handle_CHLD {
 	my $status = $?;
 	for my $tries (1 .. 3) {
 	    local $?;
-	    $pid = CORE::waitpid -1, WNOHANG;
+	    $pid = CORE::waitpid -1, &WNOHANG;
 	    $status = $?;
 	    last if isValidPid($pid);
 	}
